==> code-movie-0.0.7.tgz: package/docs/tutorial/Readme.md <==
# Tutorial

In this tutorial, we are going to create the following mighty morphing JSON animation:

<iframe height="300" style="width: 100%;" scrolling="no" title="Code.Movie Tutorial V1" src="https://codepen.io/SirPepe/embed/JjqpZae?default-tab=html%2Cresult" frameborder="no" loading="lazy" allowtransparency="true" allowfullscreen="true">
  See the Pen <a href="https://codepen.io/SirPepe/pen/JjqpZae">
  Code.Movie Tutorial V1</a> by Peter Kröner (<a href="https://codepen.io/SirPepe">@SirPepe</a>)
  on <a href="https://codepen.io">CodePen</a>.
</iframe>

<small>[View on CodePen](https://codepen.io/SirPepe/pen/JjqpZae?editors=0010)</small>

This is a somewhat modest example, but the process is essentially the same for other programming languages and larger, more complex animations.

## Get the library

The library is written in TypeScript, but can also be used with plain JavaScript. It works in browsers as well as in server-side environments. You *can* use your preferred package manager to install the library...

```shell
$ npm install @code-movie/code-movie # adjust this for use with yarn etc.
```

... but for particularly easy use in web browsers, you can also load the library from a CDN: [https://cdn.jsdelivr.net/npm/@codemovie/code-movie](https://cdn.jsdelivr.net/npm/@codemovie/code-movie)

## Load the modules

Creating an animation involves two steps:

1. Turning frames code into animation information (a **Scene object**)
2. Turning the animation information into something that actually moves, usually a **string of HTML**

To do that, there's two functions to load from the module: `fromStringsToScene` and `toAnimationHTML`. Assuming we are using a web browser, we import both functions from `@code-movie/code-movie` in a module:

```html
<!doctype html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <title>Tutorial</title>
  <script type="module">
    import { fromStringsToScene, toAnimationHTML } from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie";
  </script>
</head>
<body></body>
</html>
```

## Create the scene

To create a scene, we first need a list of **keyframe objects** that our code animation steps through. This list is an array of objects like this:

```javascript
let keyframes = [
  {
    code: `[]`,
    ranges: [], // ignore this for now
    decorations: [], // ignore this for now
  },
  {
    code: `["World"]`,
    ranges: [],
    decorations: [],
  },
  {
    code: `["Hello", "World"]`,
    ranges: [],
    decorations: [],
  },
  {
    code: `[\n  "Hello",\n  "World"\n]`,
    ranges: [],
    decorations: [],
  },
];
```

This array then gets passed into the `fromStringsToScene` function alongside our choice of tab size and the language we want to use:

```javascript
import { fromStringsToScene, toAnimationHTML } from "xxx";

let keyframes = [ /* keyframes go here */ ];

let scene = fromStringsToScene(keyframes, {
  tabSize: 2,
  language: "json",
});
```

`fromStringsToScene` parses the code, computes the animation and finally returns a **scene object** that contains all the animation information.

## Rendering the animation

A scene contains nothing but data, which is not much to look at. Luckily we can quickly turn the scene into an HTML animation by passing it into `toAnimationHTML`:

```javascript
import { fromStringsToScene, toAnimationHTML } from "xxx";

let keyframes = [ /* keyframes go here */ ];

let scene = fromStringsToScene(keyframes, {
  tabSize: 2,
  language: "json",
});

let html = toAnimationHTML(scene); // can take more options, ignore for now
```

The resulting HTML string is *huge* and contains everything a browser needs to display the animation. The animation is advanced by switching between the classes `frame0`, `frame1`, `frameN` etc. on the element with the class `cm-animation` contained in the HTML, which by itself is not very user-friendly. We can add the custom element [`<code-movie-runtime>`](https://github.com/CodeMovie/code-movie-runtime) to our code to get a minimal user interface without much effort. Wrap the HTML returned by `toAnimationHTML` in `<code-movie-runtime>` tags, dump it into the page and we are done!

```javascript
import { fromStringsToScene, toAnimationHTML } from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie";

let keyframes = [ /* keyframes go here */ ];

let scene = fromStringsToScene(keyframes, {
  tabSize: 2,
  language: "json",
});

let html = toAnimationHTML(scene);

// Load the custom element
import "https://cdn.jsdelivr.net/npm/@codemovie/code-movie-runtime";

// Wrap the generated HTML and use it in the page
document.body.innerHTML = `<code-movie-runtime controls keyframes="0 1 2 3">${html}</code-movie-runtime>`;
```

## Next steps

You can check out the [styling options](../styling/Readme.md) or read up on [API details](../animations/Readme.md) next.

==> code-movie-0.0.8.tgz: package/docs/tutorial/Readme.md <==
# Tutorial

In this tutorial, we are going to create the following mighty morphing JSON animation:

<iframe height="300" style="width: 100%;" scrolling="no" title="Code.Movie Tutorial V1" src="https://codepen.io/SirPepe/embed/JjqpZae?default-tab=html%2Cresult" frameborder="no" loading="lazy" allowtransparency="true" allowfullscreen="true">
  See the Pen <a href="https://codepen.io/SirPepe/pen/JjqpZae">
  Code.Movie Tutorial V1</a> by Peter Kröner (<a href="https://codepen.io/SirPepe">@SirPepe</a>)
  on <a href="https://codepen.io">CodePen</a>.
</iframe>

<small>[View on CodePen](https://codepen.io/SirPepe/pen/JjqpZae?editors=0010)</small>

This is a somewhat modest example, but the process is essentially the same for other programming languages and larger, more complex animations.

## Get the library

The library is written in TypeScript, but can also be used with plain JavaScript. It works in browsers as well as in server-side environments. You *can* use your preferred package manager to install the library...

```shell
$ npm install @code-movie/code-movie # adjust this for use with yarn etc.
```

... but for particularly easy use in web browsers, you can also load the library from a CDN: [https://cdn.jsdelivr.net/npm/@codemovie/code-movie](https://cdn.jsdelivr.net/npm/@codemovie/code-movie)

## Load the modules

Creating an animation involves two steps:

1. Turning frames code into animation information (a **Scene object**)
2. Turning the animation information into something that actually moves, usually a **string of HTML**

To do that, there's two functions to load from the module: `fromStringsToScene` and `toAnimationHTML`. Assuming we are using a web browser, we import both functions from `@code-movie/code-movie` in a module:

```html
<!doctype html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <title>Tutorial</title>
  <script type="module">
    import {
      fromStringsToScene,
      toAnimationHTML,
    } from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie";
  </script>
</head>
<body></body>
</html>
```

## Create the scene

To create a scene, we first need a list of **keyframe objects** that our code animation steps through. This list is an array of objects like this:

```javascript
let keyframes = [
  {
    code: `[]`,
    ranges: [], // ignore this for now
    decorations: [], // ignore this for now
  },
  {
    code: `["World"]`,
    ranges: [],
    decorations: [],
  },
  {
    code: `["Hello", "World"]`,
    ranges: [],
    decorations: [],
  },
  {
    code: `[\n  "Hello",\n  "World"\n]`,
    ranges: [],
    decorations: [],
  },
];
```

This array then gets passed into the `fromStringsToScene` function alongside our choice of tab size and the language we want to use:

```javascript
import {
  fromStringsToScene,
  toAnimationHTML,
} from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie";

let keyframes = [ /* keyframes go here */ ];

let scene = fromStringsToScene(keyframes, {
  tabSize: 2,
  language: "json",
});
```

`fromStringsToScene` parses the code, computes the animation and finally returns a **scene object** that contains all the animation information.

## Rendering the animation

A scene contains nothing but data, which is not much to look at. Luckily we can quickly turn the scene into an HTML animation by passing it into `toAnimationHTML`:

```javascript
import {
  fromStringsToScene,
  toAnimationHTML,
} from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie";

let keyframes = [ /* keyframes go here */ ];

let scene = fromStringsToScene(keyframes, {
  tabSize: 2,
  language: "json",
});

let html = toAnimationHTML(scene);
// can take more options, ignore for now
```

The resulting HTML string is *huge* and contains everything a browser needs to display the animation. The animation is advanced by switching between the classes `frame0`, `frame1`, `frameN` etc. on the element with the class `cm-animation` contained in the HTML, which by itself is not very user-friendly. We can add the custom element [`<code-movie-runtime>`](https://github.com/CodeMovie/code-movie-runtime) to our code to get a minimal user interface without much effort. Wrap the HTML returned by `toAnimationHTML` in `<code-movie-runtime>` tags, dump it into the page and we are done!

```javascript
import {
  fromStringsToScene,
  toAnimationHTML,
} from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie";

let keyframes = [ /* keyframes go here */ ];

let scene = fromStringsToScene(keyframes, {
  tabSize: 2,
  language: "json",
});

let html = toAnimationHTML(scene);

// Load the custom element
import "https://cdn.jsdelivr.net/npm/@codemovie/code-movie-runtime";

// Wrap the generated HTML and use it in the page!
document.body.innerHTML = `<code-movie-runtime controls keyframes="0 1 2 3">
${html}
</code-movie-runtime>`;
```

## Next steps

You can check out the [styling options](../styling/Readme.md) or read up on [API details](../animations/Readme.md) next.
